/* eslint-disable prettier/prettier */

import "../native";
import { Component } from "react";
import { Alert, NativeEventEmitter } from "react-native";
import JitsiMeetJS from "../lib/lib-jitsi-meet.min";
import data from "../actions";
import AudioMode from "../AudioMode";
import Logger from "./Logger";

((global) => {
  if (typeof global.$ === "undefined") {
    const jQuery = require("jquery");
    jQuery(global);
    global.$ = jQuery;
  }
})(global || window || this);
const TAG = "JitsiManager";
export default class JitsiManager extends Component {
  static localVideoUrl = "";

  static roomName = null;

  static dispatch = null;

  static region = null;

  static isAudioOnly = null;

  static onMuteAudio() {
    JitsiManager.localTracks[0].mute();
  }

  static onUnmuteAudio() {
    JitsiManager.localTracks[0].unmute();
  }

  static onMuteVideo() {
    JitsiManager.localTracks[1].mute();
  }

  static onUnmuteVideo() {
    JitsiManager.localTracks[1].unmute();
  }

  static setAudioModeSpeaker() {
    AudioMode.setAudoSourceSpeaker();
  }

  static setAudioModeEarpiece() {
    AudioMode.setAudoSourceEarPiece();
  }

  static connection = null;

  static room = null;

  static localTracks = [];

  static isJoined = true;

  static userObject;

  static receiverObject;

  static initiatorObject;

  static isInitiator;

  static users = [];

  static removedTracks = [];
  static MAX_VIDEO_STREAM_LIMIT = 1;

  static remoteTracks = {};

  static confOptions = {
    openBridgeChannel: true,
    channelLastN: JitsiManager.MAX_VIDEO_STREAM_LIMIT,
  };

  static initOptions = {
    disableAudioLevels: true,
    desktopSharingChromeExtId: "mbocklcggfhnbahlnepmldehdhpjfcjp",
    desktopSharingChromeDisabled: false,
    desktopSharingChromeSources: ["screen", "window"],
    desktopSharingChromeMinExtVersion: "0.1",
    desktopSharingFirefoxDisabled: true,
  };

  static getDomain() {
    var domain = "rtc-" + JitsiManager.region + ".cometchat.io";
    Logger.log("RTC Domain = ", domain);
    return domain;
  }

  static getMUC() {
    var muc = "conference.rtc-" + JitsiManager.region + ".cometchat.io";
    Logger.log("RTC MUC = ", muc);
    return muc;
  }

  static getBosh() {
    var bosh = "https://rtc-" + JitsiManager.region + ".cometchat.io/http-bind";
    Logger.log("RTC Bosh = ", bosh);
    return bosh;
  }
  static eventEmitter = new NativeEventEmitter(AudioMode);

  static setCallObject(user, receiver, initiator, isInitiator) {
    JitsiManager.isInitiator = isInitiator;
    JitsiManager.receiverObject = receiver;
    JitsiManager.userObject = user;
    JitsiManager.initiatorObject = initiator;
  }

  static setRegion(region) {
    JitsiManager.region = region;
  }

  static setAudioOnly(auioOnly) {
    JitsiManager.isAudioOnly = auioOnly;
  }

  static onConnectionSuccess() {
    Logger.log("onConnectionSuccess Called");
    JitsiManager.room = JitsiManager.connection.initJitsiConference(
      JitsiManager.roomName,
      JitsiManager.confOptions
    );
    Logger.log("Jitsi room User ID: ", JitsiManager.room.myUserId());
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.TRACK_ADDED,
      JitsiManager.onRemoteTrack
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.TRACK_REMOVED,
      JitsiManager.onTrackRemoved
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.CONFERENCE_JOINED,
      JitsiManager.onConferenceJoined
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.USER_JOINED,
      JitsiManager.onUserJoined
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.USER_LEFT,
      JitsiManager.onUserLeft
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.TRACK_MUTE_CHANGED,
      function (track) {
        console.info("track");
      }
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.DISPLAY_NAME_CHANGED,
      function (userID, displayName) {
        console.info("displayName");
      }
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.TRACK_AUDIO_LEVEL_CHANGED,
      function (userID, audioLevel) {
        console.info("audioLevel");
      }
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.LAST_N_ENDPOINTS_CHANGED,
      JitsiManager.onLastNEndPointChanged
    );
    JitsiManager.room.on(
      JitsiMeetJS.events.conference.DOMINANT_SPEAKER_CHANGED,
      JitsiManager.onDominantSpeakerChanged
    );

    JitsiManager.room.addCommandListener("RTCUserUid", function (data) {
      var i;
      var userExists = false;
      for (i = 0; i < JitsiManager.users.length; i++) {
        if (data.attributes.trackid == JitsiManager.users[i].trackid) {
          userExists = true;
        }
      }
      if (!userExists) {
        JitsiManager.users.push(data.attributes);
      }
    });
    JitsiManager.room.setDisplayName(JitsiManager.userObject.name);
    JitsiManager.room.join();
  }

  static onLastNEndPointChanged(leavingIds, enteringIds) {
    Logger.log(TAG, "LastN leavingIds ", leavingIds);
    Logger.log(TAG, "LastN enteringIds ", enteringIds);
  }

  static onDominantSpeakerChanged(id) {
    Logger.log(TAG, "onDominantSpeakerChanged ", id);
    JitsiManager.dispatch(data.updateDominantVideoId(id));
  }

  static onUserJoined(id, remoteUser) {
    JitsiManager.eventEmitter.emit("CallEvent", {
      type: "onConnectionSuccess",
    });
    JitsiManager.room.sendCommand("RTCUserUid", {
      attributes: {
        uid: JitsiManager.userObject.uid,
        name: JitsiManager.userObject.name,
        avatar: JitsiManager.userObject.avatar,
        trackid: JitsiManager.room.myUserId(),
      },
    });
    var user = {
      id: remoteUser.getId(),
      name: remoteUser.getDisplayName(),
      status: remoteUser._connectionStatus,
      isAudioMuted: undefined,
      isVideoMuted: undefined,
      videoTrack: undefined,
      audioTrack: undefined,
    };
    Logger.log(TAG, "onUserJoined UserObject = ", user);
    JitsiManager.dispatch(data.addUser(user));
  }

  static onUserLeft = function (id) {
    Logger.log("onUserLeft id lol : ", id);
    JitsiManager.dispatch(data.removeUser(id));
    if (!JitsiManager.remoteTracks[id]) {
      return;
    }
    JitsiManager.dispatch(data.removeRemoteTrack(id));
  };

  static onLocalTracks = function (tracks) {
    JitsiManager.localTracks = tracks;
    JitsiManager.dispatch(data.addLocalTracks(JitsiManager.localTracks));
    for (let i = 0; i < JitsiManager.localTracks.length; i++) {
      JitsiManager.localTracks[i].addEventListener(
        JitsiMeetJS.events.track.TRACK_AUDIO_LEVEL_CHANGED,
        (audioLevel) => console.info("audioLevel")
      );
      JitsiManager.localTracks[i].addEventListener(
        JitsiMeetJS.events.track.TRACK_MUTE_CHANGED,
        function () {
          console.info("local track muted");
        }
      );
      JitsiManager.localTracks[i].addEventListener(
        JitsiMeetJS.events.track.LOCAL_TRACK_STOPPED,
        function () {
          console.info("track stopped");
        }
      );
      JitsiManager.localTracks[i].addEventListener(
        JitsiMeetJS.events.track.TRACK_AUDIO_OUTPUT_CHANGED,
        (deviceId) => console.info("deviceId")
      );
      if (JitsiManager.localTracks[i].getType() === "video") {
        JitsiManager.dispatch(
          data.addLocalUrl(
            JitsiManager.localTracks[i].getOriginalStream().toURL()
          )
        );
        JitsiManager.dispatch(
          data.addLocalTrackId(JitsiManager.localTracks[i].ownerEndpointId)
        );
        Logger.log(
          "Local tracks ownerEndpointId ",
          JitsiManager.localTracks[i].ownerEndpointId
        );
      }
    }
  };

  static onTrackRemoved = function (track) {
    var id = track.ownerEndpointId;
    if (track.isLocal()) {
      return;
    }
    // if (!JitsiManager.remoteTracks[id]) {
    //     return;
    // }
    JitsiManager.removedTracks.push(id);
    JitsiManager.dispatch(data.removeRemoteTrack(id));
  };

  static onRemoteTrack = function (track) {
    if (track.isLocal()) {
      return;
    }
    Logger.log(TAG, "onRemoteTrack Called..... ", track);
    // if(JitsiManager.removedTracks.indexOf(track.ownerEndpointId) > -1){
    //     Logger.log("onRemoteTrack adding removed track ");
    //     return;
    // }
    JitsiManager.dispatch(data.addRemoteTracks(track));
    const participant = track.getParticipantId();
    if (!JitsiManager.remoteTracks[participant]) {
      JitsiManager.remoteTracks[participant] = [];
    }
    const idx = JitsiManager.remoteTracks[participant].push(track);
    track.addEventListener(
      JitsiMeetJS.events.track.TRACK_AUDIO_LEVEL_CHANGED,
      (audioLevel) => console.info("audioLevel")
    );
    track.addEventListener(
      JitsiMeetJS.events.track.TRACK_MUTE_CHANGED,
      function () {
        console.info("");
      }
    );
    track.addEventListener(
      JitsiMeetJS.events.track.LOCAL_TRACK_STOPPED,
      function () {
        console.info("");
      }
    );
    track.addEventListener(
      JitsiMeetJS.events.track.TRACK_AUDIO_OUTPUT_CHANGED,
      (deviceId) => console.info("deviceId")
    );
    const id = participant + track.getType() + idx;
    if (track.getType() === "video") {
      JitsiManager.dispatch(
        data.addRemoteUrl(track.getOriginalStream().toURL())
      );
    } else {
      //$('.remoteVideo').append(`<audio autoplay='1' id='${participant}audio${idx}' />`);
    }
    //track.attach($('#'+id)[0]);
  };

  static onConferenceJoined = function () {
    JitsiManager.isJoined = true;
    Logger.log(TAG, "Local Track to be published = ", JitsiManager.localTracks);
    for (let i = 0; i < JitsiManager.localTracks.length; i++) {
      JitsiManager.room.addTrack(JitsiManager.localTracks[i]);
    }
  };

  static onConnectionFailed = function () {
    Alert.alert(
      "Connection Failed",
      "There was a problem reaching our server. Please Try Again Later.",
      [
        {
          text: "OK",
          onPress: function () {
            JitsiManager.dispatch(data.connectionFailed(1));
          },
        },
      ],
      { cancelable: false }
    );
  };

  static init = function (room) {
    Logger.log("Init called");
    JitsiManager.users = [];
    JitsiManager.roomName = room;
    JitsiMeetJS.init(JitsiManager.initOptions);
    JitsiMeetJS.setLogLevel(JitsiMeetJS.logLevels.ERROR);
  };

  static makeConnection = function () {
    var options = {
      hosts: {
        domain: JitsiManager.getDomain(),
        muc: JitsiManager.getMUC(),
      },
      p2p: {
        enabled: false,
       
       
      },
      bosh: JitsiManager.getBosh(),
      clientNode: "http://jitsi.org/jitsimeet",
    };
    JitsiManager.connection = new JitsiMeetJS.JitsiConnection(
      null,
      null,
      options
    );
    JitsiManager.connection.connect();
  };

  static disconnect = function () {
    this.connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
      JitsiManager.onConnectionSuccess
    );
    this.connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_FAILED,
      JitsiManager.onConnectionFailed
    );
    this.connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
      JitsiManager.disconnect
    );
  };

  static addEventListener = function () {
    this.connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
      JitsiManager.onConnectionSuccess
    );
    this.connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_FAILED,
      JitsiManager.onConnectionFailed
    );
    this.connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
      JitsiManager.disconnect
    );
  };

  static createLocalTracks = function (dispatcher) {
    Logger.log("Create Local track called");
    JitsiManager.dispatch = dispatcher;

    if (JitsiManager.isAudioOnly) {
      JitsiMeetJS.createLocalTracks({ devices: ["audio"] })
        .then(JitsiManager.onLocalTracks)
        .catch((error) => {
          Logger.log("Create Local Track called..");
          throw error;
        });
    } else {
      JitsiMeetJS.createLocalTracks({
        devices: ["audio", "video"],
        facingMode: "user",
      })
        .then(JitsiManager.onLocalTracks)
        .catch((error) => {
          Logger.log("Create Local Track called..");
          throw error;
        });
    }
  };

  static onSwitchCamera = function () {
    for (let i = 0; i < JitsiManager.localTracks.length; i++) {
      if (JitsiManager.localTracks[i].getType() === "video") {
        JitsiManager.localTracks[i]._switchCamera();
      }
    }
  };

  static unMount = function () {
    if (JitsiManager.room) {
      this.room.leave().catch(function (e) {
        //console.log(e)
      });
    }
  };

  static reMount = function () {
    Logger.log(TAG, "Remount function called", "");
    Logger.log(TAG, "Connection object = ", JitsiManager.connection);
  };

  static onHangUp = function () {
    Logger.log("onHangup Called");
    if (JitsiManager.room) {
      this.room.off(
        JitsiMeetJS.events.conference.TRACK_ADDED,
        JitsiManager.onRemoteTrack
      );
      this.room.off(JitsiMeetJS.events.conference.TRACK_REMOVED, (track) => {
        console.info("track");
      });
      this.room.off(
        JitsiMeetJS.events.conference.CONFERENCE_JOINED,
        JitsiManager.onConferenceJoined
      );
      this.room.off(JitsiMeetJS.events.conference.USER_JOINED, (id) => {
        JitsiManager.remoteTracks[id] = [];
      });
      this.room.off(
        JitsiMeetJS.events.conference.USER_LEFT,
        JitsiManager.onUserLeft
      );
      this.room.off(
        JitsiMeetJS.events.conference.TRACK_MUTE_CHANGED,
        (track) => {
          console.info("track");
        }
      );
      this.room.off(
        JitsiMeetJS.events.conference.DISPLAY_NAME_CHANGED,
        function (userID, displayName) {
          console.info("displayName");
        }
      );
      this.room.off(
        JitsiMeetJS.events.conference.TRACK_AUDIO_LEVEL_CHANGED,
        function (userID, audioLevel) {
          console.info("audioLevel");
        }
      );
      this.room.leave().catch(function (e) {
        //console.log(e)
      });
      this.room = null;
    }

    if (this.connection) {
      this.connection.removeEventListener(
        JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
        JitsiManager.onConnectionSuccess
      );
      this.connection.removeEventListener(
        JitsiMeetJS.events.connection.CONNECTION_FAILED,
        JitsiManager.onConnectionFailed
      );
      this.connection.removeEventListener(
        JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
        JitsiManager.disconnect
      );
      this.connection.disconnect();
      this.connection = null;
    }
    if (this.dispatch !== null) {
      this.dispatch(data.removeLocalTracks());
      this.dispatch(data.removeRemoteTracks());
      this.dispatch(data.addLocalUrl(""));
      this.dispatch(data.addRemoteUrl(""));
    }

    if (JitsiManager.localTracks) {
      for (let i = 0; i < JitsiManager.localTracks.length; i++) {
        JitsiManager.localTracks[i].dispose();
      }
    }

    if (JitsiManager.dispatch) {
      JitsiManager.dispatch(data.clearAllUsers());
    }

    if (JitsiManager.eventEmitter) {
      JitsiManager.eventEmitter.emit("CallEvent", { type: "EndCall" });
    }
  };
}
